require "box2d"

MyScene = gideros.class(Sprite)


function MyScene:init()
	self.world = b2.World.new(0, 10, true)
	
	local screenW = application:getContentWidth()
	local screenH = application:getContentHeight()
	
	self:wall(0,screenH/2,10,screenH)
	self:wall(screenW/2,0,screenW,10)
	self:wall(screenW,screenH/2,10,screenH)
	self:wall(screenW/2,screenH,screenW,10)
	
	
    self.DB = self:createMyDynamicBody(200, 50)
    self.extraLife = {}
    self.extraLife[#self.extraLife+1] = self:createExtraLife(math.random(300, 400), 100)
    self.extraLife[#self.extraLife+1] = self:createExtraLife(math.random(200, 280), 50)
    self.extraLife[#self.extraLife+1] = self:createExtraLife(math.random(50, 80), 350)
    self.extraLife[#self.extraLife+1] = self:createExtraLife(math.random(333, 444), 250)
	self.extraLife[#self.extraLife+1] = self:createExtraLife(math.random(111, 444), 460)
    self.extraLife[#self.extraLife+1] = self:createExtraLife(math.random(444, 666), 88)
    self.extraLife[#self.extraLife+1] = self:createExtraLife(math.random(555, 777), 333)
    self.extraLife[#self.extraLife+1] = self:createExtraLife(math.random(0, 480), 222)
	
	local ground = self.world:createBody({})
	local mouseJoint = nil
	
	function self:onMouseDown(event)
		if self.DB:hitTestPoint(event.x, event.y) then
			local jointDef = b2.createMouseJointDef(ground, self.DB.body, event.x, event.y, 100000)
			mouseJoint = self.world:createJoint(jointDef)
		end
	end
	
	function self:onMouseMove(event)
		if mouseJoint ~= nil then
			mouseJoint:setTarget(event.x, event.y)
		end
	end
	
	function self:onMouseUp(event)
		if mouseJoint ~= nil then
			self.world:destroyJoint(mouseJoint)
			mouseJoint = nil
		end
	end
	
	local debugDraw = b2.DebugDraw.new()
	self.world:setDebugDraw(debugDraw)
	self:addChild(debugDraw)
	
	self:addEventListener(Event.MOUSE_DOWN, self.onMouseDown, self)
	self:addEventListener(Event.MOUSE_MOVE, self.onMouseMove, self)
	self:addEventListener(Event.MOUSE_UP, self.onMouseUp, self)
	
    self:addEventListener(Event.ENTER_FRAME, self.onEnterFrame, self)
	self.world:addEventListener(Event.BEGIN_CONTACT, self.onBeginContact, self)
end

function MyScene:wall(x, y, width, height)
	local wall = Shape.new()
	
	wall:beginPath()
	
	wall:moveTo(-width/2,-height/2)
	wall:lineTo(width/2, -height/2)
	wall:lineTo(width/2, height/2)
	wall:lineTo(-width/2, height/2)
	wall:closePath()
	wall:endPath()
	wall:setPosition(x,y)
	
	local body = self.world:createBody{type = b2.STATIC_BODY}
	body:setPosition(wall:getX(), wall:getY())
	body:setAngle(wall:getRotation() * math.pi/180)
	local poly = b2.PolygonShape.new()
	poly:setAsBox(wall:getWidth()/2, wall:getHeight()/2)
	local fixture = body:createFixture{shape = poly, density = 0.1, friction = 1.0, restitution = 0.4}
	fixture:setFilterData({categoryBits = 16, maskBits = 15})
	wall.body = body
	wall.body.type = "wall"
	self:addChild(wall)
	
	return wall
end

function MyScene:createMyDynamicBody(x, y)
	local DB = Bitmap.new(Texture.new("ball.png"))
	DB:setAnchorPoint(0.5, 0.5)
	DB:setPosition(x,y)
	local radius = DB:getWidth()/2
 
	local body = self.world:createBody{type = b2.DYNAMIC_BODY, fixedRotation = true}
	body:setPosition(DB:getX(), DB:getY())
	body:setAngle(DB:getRotation() * math.pi/180)
	local circle = b2.CircleShape.new(0, 0, radius)
	local fixture = body:createFixture{shape = circle, density = 10, friction = 1.0, restitution = 0.4}
	fixture:setFilterData({categoryBits = 1, maskBits = 26})
	DB.body = body
	DB.body.type = "DB"
	self:addChild(DB)
 
	return DB
end

function MyScene:createExtraLife(x, y)
	local SB = Bitmap.new(Texture.new("life.png"))
	SB:setAnchorPoint(0.5, 0.5)
	SB:setPosition(x, y)
	local radius = SB:getWidth()/2
 
	local body = self.world:createBody{type = b2.STATIC_BODY}
	body:setPosition(SB:getX(), SB:getY())
	body:setAngle(SB:getRotation() * math.pi/180)
	local circle = b2.CircleShape.new(0, 0, radius)
	local fixture = body:createFixture{shape = circle, density = 10, friction = 1.0, restitution = 0.4, isSensor = true}
	fixture:setFilterData({categoryBits = 8, maskBits = 17})
	SB.body = body
	SB.body.type = "SB"
	SB.body.object = SB
	self:addChild(SB)
 
	return SB
end

function MyScene:onBeginContact(e)
        local fixtureA = e.fixtureA
        local fixtureB = e.fixtureB
        local bodyA = fixtureA:getBody()
        local bodyB = fixtureB:getBody()
 
	if bodyA.type and bodyB.type then
		self:removeChild(bodyB.object:removeFromParent())
			Timer.delayedCall(1, function()
			bodyB:setActive(false)
			--or remove body completely
		--	self.world:destroyBody(bodyB)
		end)
	end
end

function MyScene:onEnterFrame()
	self.world:step(1/60, 8, 3)
	--iterate through all child sprites
	for i = 1, self:getNumChildren() do
		--get specific sprite
		local sprite = self:getChildAt(i)
		-- check if sprite HAS a body (ie, physical object reference we added)
		if sprite.body then
			--update position to match box2d world object's position
			--get physical body reference
			local body = sprite.body
			--get body coordinates
			local bodyX, bodyY = body:getPosition()
			--apply coordinates to sprite
			sprite:setPosition(bodyX, bodyY)
			--apply rotation to sprite
			sprite:setRotation(body:getAngle() * 180 / math.pi)
		end
	end
end